function [WTP, WTP_diff, WTP_joint_test] = LML_wtpxm(Res, Base, TestVec, NSim)
rng(179424673)

% Function calculates mean WTP from LML model for different vectors from INPUT.Xm and tests
% whether there are significant differences between these vectors

% INPUT:
% Res is a struct with model results (e.g. Results.LML_d)
% Base is a vector with which to compare (1 x NVarM)(e.g. vector of 0, or vector of means from Xm)
% TestVec is a matrix T x NVarM where T is number of vectors to test (e.g. different treatments)
% NSim sets no. of draws used in simulation

% OUTPUT:
% WTP - matrix (NVarA-1 x 3 x T+1) with mean WTP (+ s.e and p-val) for Base and TestVec  
% WTP_diff - matrix (NVarA-1 x 3 x T) with differences of mean WTP between Base and given vector from  TestVec (matrix contains differences, Wald statistic and p-value) 
% WTP_joint_test - matrix (T x 3); testing joint significance (for all attributes) of
% differences between Base and given vector from TestVec (matrix contains Wald statistic, degrees of freedom, and p-value)
%% Settings, generating draws etc. 

%NSim = 801000;
T = size(TestVec,1);
EstimOpt = Res.EstimOpt;
NVarA = EstimOpt.NVarA;
NGrid = EstimOpt.NGrid;
WTP = zeros(NVarA-1, 3, T+1);

GridMat = Res.GridMat;
hm1 = sobolset(NVarA,'Skip',EstimOpt.HaltonSkip,'Leap',EstimOpt.HaltonLeap);
hm1 = scramble(hm1,'MatousekAffineOwen');
err_tmp = net(hm1,NSim); % this takes every point:   
err_tmp = floor((NGrid).*err_tmp)' + 1;
for i = 1:NVarA
    err_tmp(i,:) = GridMat(i,err_tmp(i,:));
end
% err_tmp = unique(Res.err','rows')'; % using the one from the model - larger NSim (NRep x NP)

N = size(err_tmp,2);
b_tmp = B_lml(err_tmp,EstimOpt); % Betas
NVar = length(Res.bhat);
bhat = Res.bhat;
I= Res.ihess;
%Xm = Res.INPUT.Xm;

BX1 = Xmfunc(Base(ones(N,1),:), err_tmp);
BX2 = zeros(size(BX1,1), size(BX1,2), T);
for i = 1:T
    BX2(:,:,i) = Xmfunc(TestVec(i*ones(N,1),:), err_tmp);
end

%% Calculating WTP

if EstimOpt.WTP_space == 0
    errX = err_tmp(1:NVarA-1,:)./err_tmp(NVarA,:);
else
    errX = err_tmp(1:NVarA-1,:);
end

h = @(b) sum(mnlquick([b_tmp; BX1],b).*errX,2);
WTP(:,1,1) = h(bhat);
H = numdiff(h,WTP(:,1,1),bhat,1,[]);
WTP(:,2:3,1) = [sqrt(diag(H*I*H')),pv(WTP(:,1,1),sqrt(diag(H*I*H')))];

for i = 1:T
    h = @(b) sum(mnlquick([b_tmp; BX2(:,:,i)],b).*errX,2);
    WTP(:,1,1+i) = h(bhat);
    H = numdiff(h,WTP(:,1,1+i),bhat,1,[]);
    WTP(:,2:3,1+i) = [sqrt(diag(H*I*H')),pv(WTP(:,1,1+i),sqrt(diag(H*I*H')))];
end
% h = @(bhat) [-sum(mnlquick(b_tmp(Indx2,:),bhat(Indx2)).*err_tmp,2); ...
%         sum(mnlquick(b_tmp(Indx2,:),bhat(Indx2)).*err_tmp,2)-sum(mnlquick(b_tmp(Indx3,:),bhat(Indx3)).*err_tmp,2); ...
%         sum(mnlquick(b_tmp(Indx3,:),bhat(Indx3)).*err_tmp,2)-sum(mnlquick(b_tmp(Indx4,:),bhat(Indx4)).*err_tmp,2)]; % Calculates differences in mean
%     hx = feval(@(b) h(b), bhat);

%% Testing differences between WTP
WTP_diff = zeros(NVarA-1, 3, T); % Difference between treatments - wald test - [Diff, Wald stat, p-value] x no. of treatments
WTP_joint_test = zeros(T,3); % joint significance for all attributes
for i = 1:T
    h = @(b) sum((mnlquick([b_tmp; BX2(:,:,i)],b)-mnlquick([b_tmp; BX1],b)).*errX,2);
    hx = h(bhat);
    WTP_diff(:,1,i) = hx;
    H = numdiff(h,WTP_diff(:,1,i),bhat,1,[]);
    
    X = H*I*H';
    W = hx'*(X\hx);
    dF = length(hx);
    p =  1 - chi2cdf(W,dF);
    WTP_joint_test(i,:) = [W, dF, p];
        
    for j = 1:NVarA-1
        hx_j = hx(j);
        H_j = H(j,:);
        X = H_j*I*H_j';
        W = hx_j'*(X\hx_j);
        dF = length(hx_j);
        p =  1 - chi2cdf(W,dF);
        WTP_diff(j,:,i) = [hx_j, W, p];
    end
end

end

function PX = mnlquick(b_GridMat,bhat)
    Fit = b_GridMat'*bhat; % NGrid x 1
    Fit(Fit > prctile(Fit,99)) = prctile(Fit,99); % censoring to account for extreme values
    Fit = exp(Fit - max(Fit));
    Fit_sum = sum(Fit);
    PX = (Fit./Fit_sum)';
end

